"""

Latte activity log analyzer

"""

import os
import json
import time

from .Config import Config

class Analyzer(object):
    """ Analyzes Latte log data """

    def __init__(self, config, args=[]):
        self.config = config
        self.logs = {}
        self.since = 0
        if len(args) == 1:
            try:
                log_time = int(args[0], 10)
                if log_time > 0:
                    self.since = time.time() - log_time
            except:
                print 'Cannot convert time argument to integer'
        elif len(args) == 2 and args[1] in ['d', 'w', 'm']:
            try:
                log_time = int(args[0], 10)
                if log_time > 0:
                    converted = True
            except:
                print 'Cannot convert time argument to integer'
            if converted:
                if args[1] == 'd':
                    log_time *= 86400 #1 day
                elif args[1] == 'w':
                    log_time *= 604800 #1 week
                elif args[1] == 'm':
                    log_time *= 2592000 #1 month
                self.since = time.time() - log_time

    def run(self):
        """ Main analyzer loop """
        self.load_logs()
        #print repr(self.logs)
        self.analyze()

    def analyze(self):
        """ Analyzes log data and prints out results """
        if self.since:
            print 'Looking for log data since %s' % time.strftime('%d %b %Y %H:%M:%S', time.gmtime(self.since))
        if not self.logs:
            print 'There is no log data'
            return False

        windows = {}
        categories = {}
        projects = {}
        totalLogs = 0
        totalTime = 0

        # Log files
        for logFileKey in self.logs.keys():
            logFile = self.logs[logFileKey]
            totalLogs += len(logFile)
            # Individual log entries
            for logKey in logFile.keys():
                log = logFile[logKey]
                totalTime += log['time']
                if not windows.has_key(logKey):
                    windows[logKey] = log['time']
                else:
                    windows[logKey] += log['time']
                if not projects.has_key(log['project']):
                    projects[log['project']] = log['time']
                else:
                    projects[log['project']] += log['time']
                if 'categories' in log.keys():
                    if not log['categories']:
                        log['categories'] = ['(Uncategorized)']
                    # Assign time to individual categories
                    for cat in log['categories']:
                        if not categories.has_key(cat):
                            categories[cat] = log['time']
                        else:
                            categories[cat] += log['time']

        print 'Total log files: %d\nTotal log entries: %d' % (len(self.logs), \
                                                              totalLogs)
        print 'Total logged time: %s' % self.normalize_time(totalTime)
        print ''
        print 'Spent time on windows:'
        sortedWindowTimes = sorted(windows.items(), \
                                   cmp=lambda x, y: cmp(x[1], y[1]), \
                                   reverse=True)
        for (window, spent) in sortedWindowTimes:
            print '- "%s" : %s' % (window, self.normalize_time(spent))

        print ''
        print 'Spent time on categories:'
        for (category, spent) in categories.items():
            print '- "%s" : %s' % (category, self.normalize_time(spent))

        print ''
        print 'Spent time on projects:'
        for (project, spent) in projects.items():
            print '- "%s" : %s' % (project, self.normalize_time(spent))


    def load_logs(self):
        """ Loads logs from log files and stores them in memory """

        # Get a list of log files available
        logsPath = os.path.join(self.config.get('app_path'), \
                                self.config.get('stats_path'))
        logFiles = os.listdir(logsPath)
        if not logFiles:
            return False

        if self.since:
            since = str(self.since)
            logFiles = filter(lambda x: x >= since, logFiles)


        # Attempt to open each file, read and parse log data
        for logFile in logFiles:
            path = os.path.join(logsPath, logFile)
            logFileHandle = open(path, 'r')
            contents = logFileHandle.read()
            # File data may be corrupted
            try:
                jsonContents = json.loads(contents)
                self.logs[logFile] = jsonContents
            except ValueError:
                continue
            logFileHandle.close()
        return True

    def normalize_time(self, seconds):
        """ Normalizes time into user-friendly form """

        if seconds <= 0:
            return '0s'

        if seconds >= 60:
            minutes = seconds / 60
            seconds = seconds % 60
            if minutes >= 60:
                hours = minutes / 60
                minutes = minutes % 60
                return '%dh%dm%ds' % (hours, minutes, seconds)
            else:
                return '%dm%ds' % (minutes, seconds)
        else:
            return '%ds' % seconds

if __name__ == '__main__':
    Analyzer(Config()).run()



"""

Time tracking class
Handles window time logging and log information storage

"""
import time
import json
import os

class TimeTracker(object):
    """

    Tracks window time and stores window information

    """

    logs = {}

    def __init__(self, configs, categorizer, projectizer):
        self._configs = configs
        self._categorizer = categorizer
        self._projectizer = projectizer
        # Load category and project configs
        self._categorizer.load_groups()
        self._projectizer.load_groups()

    def get_sleep_time(self):
        """

        Returns sleepTime config

        """
        return self._configs.getint('sleep_time')

    def get_window_time(self, window):
        """

        Returns window time information from logs

        """
        if window in self.logs.keys():
            return self.logs[window]['time']
        else:
            return None

    def get_window_stats(self, window):
        """

        Returns a dict of window statistics.
        That includes window time, category and project.

        """
        if window in self.logs.keys():
            return self.logs[window]
        else:
            return None

    def log(self, window):
        """

        Logs window time, handles assigning to projects and categories.

        """
        if not window in self.logs.keys():
            self.logs[window] = {
                'time' : 0,
                'categories' : [],
                'project' : ''
            }
            categories = self._categorizer.assign(True, window)
            if categories:
                self.logs[window]['categories'] = [c.get_title() for c in categories]
            project = self._projectizer.assign(False, window, categories)
            if project:
                self.logs[window]['project'] = project.get_title()
        self.logs[window]['time'] += self.get_sleep_time()

    def get_logs(self):
        """

        Returns whole logs dict

        """

        return self.logs

    def clear_logs(self):
        """

        Removes logs, stored in this object

        """
        self.logs = {}

    def dump_logs(self):
        """

        Dumps logs to filesystem

        """
        # Join stats system path and create it if it doesn't exist
        target_path = os.path.join(self._configs.get('app_path'),
                                   self._configs.get('stats_path'))
        if not os.path.exists(target_path):
            os.makedirs(target_path)
        # Filename for current timestamp
        filename = str(int(time.time()))
        file_path = os.path.join(target_path, filename)

        # Write logs in json format to the log file
        log_file = open(file_path, 'w')
        log_file.write(json.dumps(self.get_logs(), indent=4))
        log_file.close()

        self.clear_logs()
        return filename



"""

Assignment manager class.
It loads project/category configs and handles assignment

"""
import sys
import os
import inspect

#from latte.Projects.Project import Project

class Assigner(object):
    """

    Assignment manager class.
    Handles project/category config loading and assignment

    """

    def __init__(self, assignerType, configs):
        self._configs = configs
        self._assignerType = assignerType
        self.groups = []

    def assign(self, multiple=False, *data):
        """

        Handle assigment
        Returns object if the window has been assigned

        """
        groups = []
        for i in self.groups:
            if i.belongs(*data):
                if not multiple:
                    return i
                else:
                    groups.append(i)

        if not multiple or (multiple and not groups):
            return None
        else:
            return groups

    def load_groups(self):
        """

        Loads user-defined grouping classes for project/category assignment

        """
        path = os.path.join(self._configs.get('app_path'))
        if os.path.exists(path):
            # Append system path to attempt to import config module
            sys.path.append(path)
            try:
                # Attempt to load groups
                if self._assignerType == 'project':
                    import projects as groups
                elif self._assignerType == 'category':
                    import categories as groups
            except Exception as ex:
                print '%s file was not loaded' % self._assignerType
                print repr(ex)
            # Extract and load only classes
            for name, obj in inspect.getmembers(groups):
                if inspect.isclass(obj) and obj.__module__ == groups.__name__:
                    self.add_group(obj)

    def add_group(self, group):
        """

        Adds group object to groups list

        """

        if self._assignerType == 'project':
            from .Projects import Project as targetClass
        elif self._assignerType == 'category':
            from .Categories import Category as targetClass

        def checkForCreatedObjects(cls, groups):
            """ Checks for registered class objects """
            for group in groups:
                if isinstance(group, cls):
                    return True
            return False

        # Make sure that it extends Category or Project Abstract Base Class
        if issubclass(group, targetClass) and \
           not checkForCreatedObjects(group, self.groups):
            self.groups.append(group())
            return True
        else:
            return False



"""

Base abstract class for Category

"""

class Category(object):
    """

    Base abstract Category class

    """

    def get_title(self):
        """

        Returns category title

        """
        raise NotImplementedError

    def belongs(self, window):
        """

        Checks if window belongs to this category.
        Returns True if it belongs, false otherwise

        """
        raise NotImplementedError



#!/usr/bin/env python

"""

Latte - Linux Automatic Time Tracker

Collects window titles you are working on, categorizes them and tracks time for
each window individually. Stores log data to the filesystem.

"""

import time
import subprocess
import os
import atexit

from .TimeTracker import TimeTracker
from .Assigner import Assigner
from .Config import Config

class Latte(object):

    """

    Main application class

    """

    def __init__(self):
        self.configs = Config()
        self.categorizer = Assigner('category', self.configs)
        self.projectizer = Assigner('project', self.configs)
        self.tracker = TimeTracker(configs=self.configs,
                                   categorizer=self.categorizer,
                                   projectizer=self.projectizer)

        # Register a cleanup
        atexit.register(self.cleanup)

        # Add application configuration path
        if not os.path.exists(self.configs.get('app_path')):
            os.makedirs(self.configs.get('app_path'))

    def cleanup(self):
        """

        Atexit cleanup method. Force dumps log information to the filesystem

        """
        self.tracker.dump_logs()

    def run(self):
        """

        Primary application loop

        """
        duration = 0
        while True:
            title = get_active_window_title()
            if title:
                self.tracker.log(title)
                stats = self.tracker.get_window_stats(title)
                print title, \
                    repr(stats['categories']), \
                    repr(stats['project']), \
                    stats['time']

            time.sleep(self.configs.getint('sleep_time'))
            # Track time since last save and do autosaves
            duration += self.configs.getint('sleep_time')
            if duration >= self.configs.getint('autosave_time'):
                duration = 0
                self.tracker.dump_logs()

def get_active_window_title():
    """

    Fetches active window title using xprop

    """
    try:
        active = subprocess.Popen(["xprop", "-root", "_NET_ACTIVE_WINDOW"],
                                stdout=subprocess.PIPE)
        active_id = active.communicate()[0].strip().split()[-1]
        window = subprocess.Popen(["xprop", "-id", active_id, "WM_NAME"],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        return window.communicate()[0].strip().split('"', 1)[-1][:-1]
    except:
        return ''


if __name__ == '__main__':
    Latte().run()



"""

Config class

Handles application configuration loading

"""

import ConfigParser
import os

class Config(object):
    """ Handles config loading and parsing. """

    def __init__(self, path='~/.latte'):
        self.config = ConfigParser.ConfigParser()
        self.load_config(path)
        self.config.set('main', 'app_path', os.path.expanduser(path))

    def load_config(self, path):
        """ Attempt to load configs from default path. """
        path = os.path.expanduser(path + '/config')
        if os.path.exists(path):
            self.config.read(path)
        else:
            self.config.add_section('main')
            self.config.set('main', 'stats_path', 'stats/')
            self.config.set('main', 'sleep_time', '5')
            self.config.set('main', 'autosave_time', '3600')

    def get(self, item, section='main'):
        """ Fetches config item from the list. """
        return self.config.get(section, item)

    def getint(self, item, section='main'):
        """ Attempts to fetch config item as integer. """
        return self.config.getint(section, item)


if __name__ == '__main__':
    Config()



"""

Assigns a project according to window title and category

"""
class Project(object):
    """

    Project assignment class based on window title and category

    """

    def get_title(self):
        """

        Returns project title string

        """
        raise NotImplementedError

    def belongs(self, window, category):
        """

        Checks if window belongs to this particular project.
        Returns True if it belongs, False otherwise

        """
        raise NotImplementedError



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

config = {
    'description': 'Automatic rule based Linux Time Tracker',
    'author': 'Tautvidas Sipavicius',
    'author_email': 'flakas@tautvidas.com',
    'url': 'https://github.com/flakas/Latte',
    'install_requires': [],
    'packages': ['latte', 'latte.Categories', 'latte.Projects'],
    'scripts': ['bin/latte'],
    'version': '1.1',
    'name': 'latte'
}

setup(**config)
